'use strict';

const fs = require('fs');
const protocolify = require('protocolify');
const { reporterTypes } = require('../lib/reporter');

const defaultConfig = require('./default-config.json');

const getUrl = (testUrl) => {
    let rawUrl, urlSettings;
    if (typeof testUrl === 'object') {
        if (!testUrl.url) {
            throw new Error('Invalid configuration, url objects must have a "url" property');
        }
        rawUrl = testUrl.url;
        urlSettings = testUrl.settings;
    }
    else {
        rawUrl = testUrl;
    }
    return { rawUrl, urlSettings };
};

const processTestSetting = (setting) => {
    if (typeof setting === 'object') {
        return setting;
    }
    return { enabled: setting === 'true' || setting === true };
};

const processAllTestSettings = (settings) => {
    if (!settings) {
        return;
    }
    const processedSettings = {};
    for (const key of Object.keys(settings)) {
        processedSettings[key] = processTestSetting(settings[key]);
    }
    return processedSettings;
};

const consolidateTestSettings = (defaultSettings, globalSettings, urlSettings) => {
    const sanitizedGlobalSettings = globalSettings ? globalSettings : {};
    const sanitizedUrlSettings = urlSettings ? urlSettings : {};
    const processedSettings = {};
    for (const key of Object.keys(defaultSettings)) {
        processedSettings[key] = Object.assign({}, defaultSettings[key], sanitizedGlobalSettings[key], sanitizedUrlSettings[key]);
    }
    return processedSettings;
};

const processUrls = (config) => {
    if (!config.urls) {
        throw new Error('No test URLs specified in configuration file');
    }
    const processedUrls = [];
    const processedConfigSettings = processAllTestSettings(config.settings);
    for (const testUrl of config.urls) {
        const { rawUrl, urlSettings } = getUrl(testUrl);
        processedUrls.push({
            url: protocolify(rawUrl),
            rawUrl,
            settings: consolidateTestSettings(defaultConfig.settings, processedConfigSettings, processAllTestSettings(urlSettings))
        });
    }
    if (processedUrls.length === 0) {
        throw new Error('No test URLs found in configuration file');
    }
    return processedUrls;
};

const getHtmlHintConfig = (htmlHintConfigFilename) => {
    if (!fs.existsSync(htmlHintConfigFilename)) {
        return;
    }
    return JSON.parse(fs.readFileSync(htmlHintConfigFilename, 'utf-8'));
};

const reportersExist = (reporters) => {
    return reporters.every(reporter => Object.values(reporterTypes).includes(reporter));
};

const validateReporters = (reporters) => {
    if (!Array.isArray(reporters) || reporters.length < 1 || !reportersExist(reporters)) {
        throw new TypeError('Invalid reporters configuration property');
    }
};

const getReporters = (reporters) => {
    if (reporters) {
        validateReporters(reporters);
        return reporters;
    }
    return defaultConfig.reporters;
};

const processConfig = (configFileName, htmlHintConfigFilename) => {
    const config = JSON.parse(fs.readFileSync(configFileName, 'utf-8'));
    return {
        puppeteerLaunchOptions: config.puppeteerLaunchOptions,
        urls: processUrls(config),
        htmlHintConfig: getHtmlHintConfig(htmlHintConfigFilename),
        reporters: getReporters(config.reporters)
    };
};

module.exports = processConfig;
